% lab13_ex_instrument_virtual_model.m
% finding parameters of musical instrument harmonics
clear all; close all;

what = 1;  % 1=piano,  2=piano Cdur, 3=guitar, 4=trumpet, 5=clarinet 
           % 6=violin, 7=flute

% Read signal: n1st - first sample to analyse, Nfft - number of samples
if(what==1) [ x, fs ] = audioread('piano.wav');       n1st = 4840;   Nfft = 2^16; Kmax=11; end
if(what==2) [ x, fs ] = audioread('piano_Cdur.wav');  n1st = 5036;   Nfft = 2^16; Kmax=25; end
if(what==3) [ x, fs ] = audioread('guitar.wav');      n1st = 123590; Nfft = 2^18; Kmax=12; end
if(what==4) [ x, fs ] = audioread('trumpet.wav');     n1st = 60260;  Nfft = 2^16; Kmax=7;  end
if(what==5) [ x, fs ] = audioread('clarinet_Bb.aif'); n1st = 44945;  Nfft = 2^16; Kmax=15; end
if(what==6) [ x, fs ] = audioread('violin.wav');      n1st = 15974;  Nfft = 2^16; Kmax=14; end
if(what==7) [ x, fs ] = audioread('flute.aif');       n1st = 8501;   Nfft = 2^16; Kmax=9;  end

% PIANO
% for fundamental frequency
% fm_start = 6Hz, decrease 1Hz/sec
% depth df_start = 125/17 Hz, decrease 25/17 Hz/sec % measured for 17-th harmonic

x = x(:,1); Nx = length(x), fs,

x = x';
dt=1/fs; t=dt*(0:Nx-1);
%x = exp(-0.25*t) .* sin(2*pi*109.668*t);  % test

% Observe signal shape
figure;
subplot(211); plot(t,x,'b.-'); grid; xlabel('t [s]'); title('x(t)');
subplot(212); plot(x,'b.-'); grid; xlabel('n'); title('x(n)'); pause   % signal shape

% Observe signal spectrogram
if(1)
   Nf=2048; figure;
 % df=1; f=df*(0:5000); spectrogram(x,Nf,Nf-Nf/8,f,fs);  % Matlab (new)
   specgram(x,Nf,fs,Nf,Nf-Nf/8);                         % Octave, Matlab (old)
   pause
end

% Choose signal fragment for analysis
x = x( n1st : n1st + Nfft-1); N = length(x);

% Observe signal spectrum
df = fs/Nfft; f = df*(0:Nfft-1); k=1:Nfft/2+1;
XdB = 20*log10( abs(fft(x(1:Nfft))/Nfft) );
figure;
subplot(211); plot( k, XdB(k), 'b.-'); grid; xlabel('DFT bin k'); title('|X(f)| [dB]'); 
subplot(212); plot( f(k), XdB(k), 'b.-'); grid; xlabel('f [Hz]'); title('|X(f)| [dB]'); pause 

% Find indexes of spectra maxima
XdB = XdB( 1 : Nfft/2+1 ); XdBcopy = XdB;
indexes = zeros(1,Kmax);
[ val, ind ] = max( XdB );
indexes(1) = ind;
skip = round( 0.75*ind );
XdB(1:ind+skip) = -250*ones(1,ind+skip);
for k = 2 : Kmax
    [ val, ind ] = max( XdB );
    indexes( k ) = ind;
    XdB(ind-skip:ind+skip) = -250*ones(1,2*skip+1);
end
figure;
subplot(111);
plot( 1:Nfft/2+1, XdBcopy, 'b.-',indexes,XdBcopy(indexes),'ro'); grid;
xlabel('DFT bin k'); title('|X(f)| [dB]'); pause 

% Signal analysis
NI=3; P=2; [Om, D, A, p] = BY1_LC_multi(x,indexes,NI,P);

% Signal synthesis
Nx = N;         % only synthesized part
n = 1 : Nx;
xs = zeros(1,Nx);
for k = 1 : Kmax
    xs = xs + A(k)*cos( Om(k)*n + p(k) ).* exp(-D(k)*n);
    err1(k) = sum( (x-xs).^2 );
    err2(k) = sum( abs(x-xs) );
end
figure;
subplot(211); plot(1:Kmax,err1,'bo-'); xlabel('k'); title('ERR1(k)'); grid;
subplot(212); plot(1:Kmax,err2,'bo-'); xlabel('k'); title('ERR2(k)'); grid;
pause
figure; plot(1:Nx,x,'r-',1:Nx,xs,'b-'); title('x(t) (RED), xs(t) (BLUE)');
disp('Original'); soundsc(x,fs); pause
disp('Synthesized'); soundsc(xs,fs); pause
figure; plot(x-xs); grid; title('x(t)-xs(t)'); pause

% Two times lower
SCALE = 0.5;
xss = zeros(1,Nx);
for k = 1 : Kmax
    xss = xss + A(k)*cos( SCALE*(Om(k)*n + p(k)) ).* exp(-D(k)*n);  
end
figure; plot(1:Nx,xs,'r-',1:Nx,xss,'b-'); title('x(t) (RED), xs(t) (BLUE)');
soundsc(xss,fs); pause

% Two times higher
SCALE = 2;
xss = zeros(1,Nx);
for k = 1 : Kmax
    xss = xss + A(k)*cos( SCALE*(Om(k)*n + p(k)) ).* exp(-D(k)*n);  
end
figure; plot(1:Nx,xs,'r-',1:Nx,xss,'b-'); title('x(t) (RED), xs(t) (BLUE)');
soundsc(xss,fs); pause

AOPD = [ A' Om' p' D'];
disp('Amplitude | Omega | Phase | Damping');
AOPD

% Playing musical gamma using virtual piano

[Kmax, ~] = size( AOPD );
harm = round( AOPD(:,2)/AOPD(1,2) ); % pause

fs = 44100;     % sampling frequency
fmax = 12000;   % maximum frequency of the fundamental frequecy harmonic
K = 2;          % 1/K seconds for one note

n = 1 : fs/K; Nx = length(n);
for oct=0:8                  % 0:8 = 9 octaves
    foct = 2^oct * 27.5;
    for note=0:11;           % 12 notes in each
        focno = foct * 2^(note/12);
        xs = zeros(1,Nx);
        for k = 1 : Kmax
            if( harm(k)*focno < 12000 )  % frequency band reduction
                xs = xs + AOPD(k,1) * cos( harm(k)*(2*pi*focno/fs*n + AOPD(k,3)) ) .* exp(-AOPD(k,4)*n);  
            end
        end
        sound(xs,fs); pause(1/K)         % play next note; different sound in Matlab (OK) and Octave (?)
    end
end    

% ########################################################################

xs13 = resample(xs,1,3);
soundsc(xs13,fs); pause

xs12 = resample(xs,1,2);
soundsc(xs12,fs); pause

xs21 = resample(xs,2,1);
soundsc(xs21,fs); pause

xs31 = resample(xs,3,1);
soundsc(xs31,fs); pause