%% Testing MULTI-COMPONENT damped sine signal
% In Octave load the function awgn(): >>pkg load communications[ENTER]
close all; clear all; clc

K = 3;              % number of components, in this program up-to 3
N = 500;            % number of samples
SNR = 60;           % signal to noise ratio
C = 4;              % 4! correlation version used? 1,2,3,4

%% Test signals
n = 0 : N-1;     % sample indexes
Om = [  10.2,     40.2,   80.2  ] * (2*pi)/N;
D  = [  1e-2,    1e-2,   1e-2  ];
A  = [  2.3,     1.3,    1.3   ];
p  = [ -pi/4,   -pi/5,  -pi/7 ];

x = zeros(1,N);
for k = 1 : K
    x = x + A(k) * exp(-n*D(k)) .* cos(Om(k)*n+p(k));    
end
y = awgn( x, SNR );    % Gaussian noise, SNR in [dB]; Octave: >>pkg load comunications 
figure;
subplot(211); plot(x); grid; title('x(n)'); 
subplot(212); plot(y); grid; title('y(n) = x(n) + noise'); pause

% Calculations
De = zeros(5,K); Ome = zeros(5,K);
NI = 10;
[Om1  D1] = fProny(y,K);        De(1,:) = (D1'-D)./D;  Ome(1,:) = (Om1'-Om)./Om;
[Om2, D2] = fSTMB(y,K,NI);      De(2,:) = (D2'-D)./D;  Ome(2,:) = (Om2'-Om)./Om;
[Om3, D3] = fKT(y,K);           De(3,:) = (D3'-D)./D;  Ome(3,:) = (Om3'-Om)./Om;
[Om4, D4] = fMatPen(y,K);       De(4,:) = (D4'-D)./D;  Ome(4,:) = (Om4'-Om)./Om;
[Om5, D5] = fYuleWalker(y,K,C); De(5,:) = (D5'-D)./D;  Ome(5,:) = (Om5'-Om)./Om;

methods = [ ...
'fProny     ';
'fSTMB      ';
'fKT        ';
'fMatPen    ';
'fYuleWalker' ];

disp('ERROR OMEGA [%] =============================================================');
for k = 1 : length(Ome(:,1))
    fprintf('%s:  Ome1 = %14.8f,  Ome2 = %14.8f,  Ome3 = %14.8f \n',methods(k,:),100*Ome(k,1:K));
end
disp('ERROR DAMPING [%] ===========================================================');
for k = 1 : length(Ome(:,1))
    fprintf('%s:  D1 = %14.8f,  D2 = %14.8f,  D3 = %14.8f \n',methods(k,:),100*De(k,1:K));
end


